<?php

/*********************************************************************

 PHP Blog Ver 2.09 (2009/06/08)

*********************************************************************/

/********* 外部ファイル読み込み *************************************/

require_once './smarty/Smarty.class.php';
require_once './config.php';

if (PICTOGRAM_MODE) {
	require_once PICTOGRAM_MAIN_FILE;
}

/********* メイン処理 ***********************************************/

//ユーザー情報取得
$users = user();

//セッション開始
session();

//データ正規化
normalize();

//設定読み込み
$configs = config();

//URLルーティング
$params = routing();

//データベース接続
$pdo = database();

//テンプレート設定
$smarty = template();

//プラグイン実行
plugin('page');

//モードに応じた処理を実行
switch ($_REQUEST['mode']) {
	case 'setup':
		setup();
		break;
	case 'view':
		view();
		break;
	case 'comment':
		comment();
		break;
	case 'trackback':
		trackback();
		break;
	case 'archive':
		archive();
		break;
	case 'search':
		search();
		break;
	case 'rss':
		rss();
		break;
	case 'admin':
		if (isloggedin()) {
			switch ($_REQUEST['work']) {
				case 'form':
					admin_form();
					break;
				case 'regist':
					admin_regist();
					break;
				case 'edit':
					admin_edit();
					break;
				case 'del':
					admin_del();
					break;
				case 'file':
					admin_file();
					break;
				case 'file_add':
					admin_file_add();
					break;
				case 'file_del':
					admin_file_del();
					break;
				case 'comment':
					admin_comment();
					break;
				case 'comment_del':
					admin_comment_del();
					break;
				case 'trackback':
					admin_trackback();
					break;
				case 'trackback_del':
					admin_trackback_del();
					break;
				case 'config':
					admin_config();
					break;
				case 'config_edit':
					admin_config_edit();
					break;
				case 'logout':
					admin_logout();
					break;
				default:
					admin_status();
			}
		} else {
			admin();
		}
		break;
	default:
		article();
}

exit;

/********* ユーザー定義関数 *****************************************/

/* ユーザー情報取得 */
function user()
{
	$users = array();

	//接続環境取得
	if (preg_match('/(DoCoMo|SoftBank|Vodafone|J-PHONE|UP\.Browser|DDIPOCKET|WILLCOM)/', $_SERVER['HTTP_USER_AGENT'])) {
		$users['type'] = 'mobile';
	} else {
		$users['type'] = 'pc';
	}

	//キャリア取得
	if (preg_match('/DoCoMo/', $_SERVER['HTTP_USER_AGENT'])) {
		$users['career'] = 'docomo';
	} elseif (preg_match('/(SoftBank|Vodafone|J-PHONE)/', $_SERVER['HTTP_USER_AGENT'])) {
		$users['career'] = 'softbank';
	} elseif (preg_match('/UP\.Browser/', $_SERVER['HTTP_USER_AGENT'])) {
		$users['career'] = 'au';
	} elseif (preg_match('/(DDIPOCKET|WILLCOM)/', $_SERVER['HTTP_USER_AGENT'])) {
		$users['career'] = 'willcom';
	} else {
		$users['career'] = null;
	}

	//コンテンツタイプ取得
	if (preg_match('/(DoCoMo|SoftBank|Vodafone|J-PHONE|DDIPOCKET|WILLCOM)/', $_SERVER['HTTP_USER_AGENT'])) {
		$users['content'] = 'application/xhtml+xml';
	} else {
		$users['content'] = 'text/html';
	}

	//フォームメソッド取得
	if (preg_match('/(J-PHONE|UP\.Browser)/', $_SERVER['HTTP_USER_AGENT'])) {
		$users['method'] = 'get';
	} else {
		$users['method'] = 'post';
	}

	//固体識別情報取得
	if (isset($_SERVER['HTTP_X_DCMGUID'])) {
		$users['serial'] = $_SERVER['HTTP_X_DCMGUID'];
	} elseif (isset($_SERVER['HTTP_X_UP_SUBNO'])) {
		$users['serial'] = $_SERVER['HTTP_X_UP_SUBNO'];
	} elseif (isset($_SERVER['HTTP_X_JPHONE_UID'])) {
		$users['serial'] = $_SERVER['HTTP_X_JPHONE_UID'];
	} else {
		$users['serial'] = null;
	}

	return $users;
}

/* セッション開始 */
function session()
{
	if (isset($_SERVER['HTTPS']) and $_SERVER['HTTPS'] != 'off') {
		$info = parse_url(HTTPS_PATH);
	} else {
		$info = parse_url(HTTP_PATH);
	}

	session_set_cookie_params(SESSION_LIFETIME, $info['path']);
	session_cache_limiter(SESSION_CACHE);
	session_start();

	return;
}

/* 受信データ正規化 */
function normalize()
{
	global $users;

	//不正データ削除
	$_GET     = sanitize($_GET);
	$_POST    = sanitize($_POST);
	$_REQUEST = sanitize($_REQUEST);
	$_SERVER  = sanitize($_SERVER);
	$_COOKIE  = sanitize($_COOKIE);

	//改行コード統一
	$_GET     = unify($_GET);
	$_POST    = unify($_POST);
	$_REQUEST = unify($_REQUEST);
	$_SERVER  = unify($_SERVER);
	$_COOKIE  = unify($_COOKIE);

	//絵文字コード統一
	if (PICTOGRAM_MODE) {
		$_GET  = unify_pictogram($_GET);
		$_POST = unify_pictogram($_POST);
	}

	//文字コード変換
	if ($users['career'] == 'au') {
		$_GET     = convert($_GET);
		$_POST    = convert($_POST);
		$_REQUEST = convert($_REQUEST);
		$_SERVER  = convert($_SERVER);
	}

	//GETデータ取得
	if ($users['method'] == 'get') {
		$_POST = $_GET;
	}

	return;
}

/* 設定読み込み */
function config()
{
	$configs = array();

	if ($dir = scandir(CONFIG_DIR)) {
		foreach ($dir as $entry) {
			if (!is_file(CONFIG_DIR . $entry) or !preg_match('/^(\w+)\.ini$/', $entry, $matches)) {
				continue;
			}

			$config = parse_ini_file(CONFIG_DIR . $entry);

			foreach ($config as $key => $value) {
				$value = str_replace('\\\\', "\a", $value);
				$value = str_replace('\n', "\n", $value);
				$value = str_replace("\a", '\\', $value);

				$configs[$matches[1]][$key] = $value;
			}
		}
	} else {
		exit('設定ファイル格納ディレクトリを開けません。');
	}

	return $configs;
}

/* URLルーティング */
function routing()
{
	$request_uri = explode('/', strtok($_SERVER['REQUEST_URI'], '?'));
	$script_name = explode('/', $_SERVER['SCRIPT_NAME']);

	for ($i = 0; $i < sizeof($script_name); $i++) {
		if ($request_uri[$i] == $script_name[$i]) {
			unset($request_uri[$i]);
		}
	}
	$params = array_values(array_map('urldecode', $request_uri));

	if (isset($params[0]) and preg_match('/\D/', $params[0])) {
		if (!isset($_REQUEST['mode'])) {
			$_REQUEST['mode'] = $params[0];
		}
		if (!isset($_REQUEST['work'])) {
			$_REQUEST['work'] = '';
		}
	} else {
		if (!isset($_REQUEST['mode'])) {
			$_REQUEST['mode'] = '';
		}
		if (!isset($_REQUEST['work'])) {
			$_REQUEST['work'] = '';
		}
	}

	return $params;
}

/* データベース接続 */
function database()
{
	if (DATABASE_TYPE == 'sqlite3') {
		$dsn = 'sqlite:' . DATABASE_NAME;
	} elseif (DATABASE_TYPE == 'sqlite2') {
		$dsn = 'sqlite2:' . DATABASE_NAME;
	} else {
		$dsn = 'mysql:dbname=' . DATABASE_NAME . ';host=' . DATABASE_HOST;
	}

	try {
		$pdo = new PDO(
			$dsn,
			DATABASE_USER,
			DATABASE_PWD,
			array(
				PDO::ATTR_ERRMODE => PDO::ERRMODE_SILENT
			)
		);
	} catch (PDOException $e) {
		exit($e->getMessage());
	}

	if (DATABASE_TYPE == 'mysql' and DATABASE_CHARSET) {
		$stmt = $pdo->query('SET NAMES ' . DATABASE_CHARSET);
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}
	}

	return $pdo;
}

/* テンプレート設定 */
function template()
{
	global $users, $configs;

	$smarty = new Smarty();
	$smarty->template_dir = TEMPLATE_DIR;
	$smarty->compile_dir  = TEMPLATE_COMPILE;

	$smarty->default_modifiers = array('escape');

	if (REWRITE_MODE) {
		$http_url  = preg_replace('/\/$/', '', HTTP_PATH);
		$https_url = preg_replace('/\/$/', '', HTTPS_PATH ? HTTPS_PATH : HTTP_PATH);
	} else {
		$http_url  = HTTP_PATH . MAIN_FILE;
		$https_url = (HTTPS_PATH ? HTTPS_PATH : HTTP_PATH) . MAIN_FILE;
	}

	if ($users['serial'] and ADMIN_SERIAL and in_array($users['serial'], explode(',', ADMIN_SERIAL))) {
		$login = true;
	} elseif (!empty($_SESSION['login']) or !empty($_COOKIE['session'])) {
		$login = true;
	} else {
		$login = false;
	}

	$smarty->assign('info', array(
		'session_name' => session_name(),
		'session_id'   => session_id(),
		'title'        => $configs['_blog']['title'],
		'description'  => $configs['_blog']['description'],
		'back_url'     => $configs['_blog']['back_url'],
		'configs'      => $configs,
		'http_path'    => HTTP_PATH,
		'https_path'   => HTTPS_PATH ? HTTPS_PATH : HTTP_PATH,
		'main_file'    => MAIN_FILE,
		'file_dir'     => FILE_DIR,
		'http_url'     => $http_url,
		'https_url'    => $https_url,
		'login'        => $login,
		'type'         => $users['type'],
		'career'       => $users['career'],
		'content'      => $users['content'],
		'method'       => $users['method'],
		'serial'       => $users['serial']
	));

	return $smarty;
}

/* プラグイン実行 */
function plugin($type)
{
	if ($dir = scandir(PLUGIN_DIR)) {
		foreach ($dir as $entry) {
			if (!is_file(PLUGIN_DIR . $entry) or !preg_match("/^$type\.\w+\.php$/", $entry)) {
				continue;
			}

			include_once PLUGIN_DIR . $entry;
		}
	} else {
		exit('プラグイン格納ディレクトリを開けません。');
	}

	return;
}

/* セットアップ */
function setup()
{
	global $users, $configs, $params, $pdo, $smarty;

	//記事格納テーブル作成
	if (DATABASE_TYPE == 'mysql') {
		$stmt = $pdo->query('CREATE TABLE ' . DATABASE_TABLE_ARTICLE . '(no INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY, stat INT UNSIGNED NOT NULL, break INT UNSIGNED NOT NULL, comt INT UNSIGNED NOT NULL, tb INT UNSIGNED NOT NULL, date DATETIME NOT NULL, subj VARCHAR(255) NOT NULL, text TEXT NOT NULL, tag VARCHAR(255))');
	} else {
		$stmt = $pdo->query('CREATE TABLE ' . DATABASE_TABLE_ARTICLE . '(no INTEGER PRIMARY KEY, stat INTEGER UNSIGNED NOT NULL, break INTEGER UNSIGNED NOT NULL, comt INTEGER UNSIGNED NOT NULL, tb INTEGER UNSIGNED NOT NULL, date DATETIME NOT NULL, subj VARCHAR NOT NULL, text TEXT NOT NULL, tag VARCHAR)');
	}
	if (!$stmt) {
		db_error($pdo->errorInfo());
	}

	//コメント格納テーブル作成
	if (DATABASE_TYPE == 'mysql') {
		$stmt = $pdo->query('CREATE TABLE ' . DATABASE_TABLE_COMMENT . '(no INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY, pno INT UNSIGNED NOT NULL, date DATETIME NOT NULL, name VARCHAR(80) NOT NULL, mail VARCHAR(80), url VARCHAR(255), text TEXT NOT NULL, host VARCHAR(80))');
	} else {
		$stmt = $pdo->query('CREATE TABLE ' . DATABASE_TABLE_COMMENT . '(no INTEGER PRIMARY KEY, pno INTEGER UNSIGNED NOT NULL, date DATETIME NOT NULL, name VARCHAR NOT NULL, mail VARCHAR, url VARCHAR, text TEXT NOT NULL, host VARCHAR)');
	}
	if (!$stmt) {
		db_error($pdo->errorInfo());
	}

	//トラックバック格納テーブル作成
	if (DATABASE_TYPE == 'mysql') {
		$stmt = $pdo->query('CREATE TABLE ' . DATABASE_TABLE_TRACKBACK . '(no INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY, pno INT UNSIGNED NOT NULL, date DATETIME NOT NULL, name VARCHAR(80) NOT NULL, title VARCHAR(255) NOT NULL, url VARCHAR(255) NOT NULL, text TEXT, host VARCHAR(80))');
	} else {
		$stmt = $pdo->query('CREATE TABLE ' . DATABASE_TABLE_TRACKBACK . '(no INTEGER PRIMARY KEY, pno INTEGER UNSIGNED NOT NULL, date DATETIME NOT NULL, name VARCHAR NOT NULL, title VARCHAR NOT NULL, url VARCHAR NOT NULL, text TEXT, host VARCHAR)');
	}
	if (!$stmt) {
		db_error($pdo->errorInfo());
	}

	//完了画面表示
	echo "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
	echo "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n";
	echo "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"ja\" lang=\"ja\" dir=\"ltr\">\n";
	echo "<head>\n";
	echo "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />\n";
	echo "<title>Complete</title>\n";
	echo "</head>\n";
	echo "<body>\n";
	echo "<p>セットアップが完了しました。</p>\n";
	echo "</body>\n";
	echo "</html>\n";

	return;
}

/* 記事一覧 */
function article()
{
	global $users, $configs, $params, $pdo, $smarty;

	//表示ページ数取得
	if (isset($params[0])) {
		$page = $params[0];
	} else {
		$page = isset($_GET['page']) ? $_GET['page'] : 0;
	}

	//プラグイン実行
	plugin('begin');

	//キャッシュ設定
	if (CACHE_ARTICLE and !$page) {
		$smarty->caching        = 1;
		$smarty->compile_check  = false;
		$smarty->cache_dir      = CACHE_DIR;
		$smarty->cache_lifetime = CACHE_LIFETIME;
	}

	if (!$smarty->is_cached(TEMPLATE_ARTICLE) or $page) {
		//表示記事番号取得
		$stmt = $pdo->prepare('SELECT no FROM ' . DATABASE_TABLE_ARTICLE . ' WHERE stat = 1 ORDER BY date DESC LIMIT :start, :limit');
		$stmt->bindValue(':start', intval($configs['_blog']['page_size']) * $page, PDO::PARAM_INT);
		$stmt->bindValue(':limit', intval($configs['_blog']['page_size']), PDO::PARAM_INT);
		$flag = $stmt->execute();
		if (!$flag) {
			db_error($stmt->errorInfo());
		}

		$shows = array();
		while ($data = $stmt->fetch(PDO::FETCH_NUM)) {
			$shows[] = $data[0];
		}

		//コメント数取得
		$comments = array();
		if (!empty($shows)) {
			$stmt = $pdo->query('SELECT pno, COUNT(*) FROM ' . DATABASE_TABLE_COMMENT . ' WHERE pno IN(' . implode(',', $shows) . ') GROUP BY pno');
			if (!$stmt) {
				db_error($pdo->errorInfo());
			}

			while ($data = $stmt->fetch(PDO::FETCH_NUM)) {
				$comments[$data[0]] = $data[1];
			}
		}

		//トラックバック数取得
		$trackbacks = array();
		if (!empty($shows)) {
			$stmt = $pdo->query('SELECT pno, COUNT(*) FROM ' . DATABASE_TABLE_TRACKBACK . ' WHERE pno IN(' . implode(',', $shows) . ') GROUP BY pno');
			if (!$stmt) {
				db_error($pdo->errorInfo());
			}

			while ($data = $stmt->fetch(PDO::FETCH_NUM)) {
				$trackbacks[$data[0]] = $data[1];
			}
		}

		//記事取得
		if (!empty($shows)) {
			$stmt = $pdo->query('SELECT * FROM ' . DATABASE_TABLE_ARTICLE . ' WHERE no IN(' . implode(',', $shows) . ') ORDER BY date DESC');
			if (!$stmt) {
				db_error($pdo->errorInfo());
			}

			$articles = array();
			while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
				if ($data['break']) {
					$data['text'] = '<p>' . nl2br($data['text']) . '</p>';
				}

				$articles[] = array(
					'no'        => $data['no'],
					'break'     => $data['break'],
					'comt'      => $data['comt'],
					'tb'        => $data['tb'],
					'date'      => $data['date'],
					'subj'      => $data['subj'],
					'text'      => $data['text'],
					'tag'       => create_tag($data['tag']),
					'comment'   => isset($comments[$data['no']])   ? $comments[$data['no']]   : 0,
					'trackback' => isset($trackbacks[$data['no']]) ? $trackbacks[$data['no']] : 0
				);
			}
			$smarty->assign('articles', $articles);
		}

		//記事数取得
		$stmt = $pdo->query('SELECT COUNT(*) FROM ' . DATABASE_TABLE_ARTICLE . ' WHERE stat = 1');
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}

		$data  = $stmt->fetch(PDO::FETCH_NUM);
		$count = $data[0];

		$smarty->assign('count', $count);

		//ページ前後移動
		if ($page > 0) {
			$smarty->assign('prev', array(
				'id'   => $page - 1,
				'flag' => true
			));
		}
		if (intval(($count - 1) / $configs['_blog']['page_size']) > $page) {
			$smarty->assign('next', array(
				'id'   => $page + 1,
				'flag' => true
			));
		}

		//ページ直接移動
		$pages = array();
		for ($i = 0; $i <= intval(($count - 1) / $configs['_blog']['page_size']); $i++) {
			$pages[] = array(
				'id'   => $i,
				'flag' => $page == $i ? true : false
			);
		}
		$smarty->assign('pages', $pages);
	}

	//プラグイン実行
	plugin('display');

	//データ表示
	output(TEMPLATE_ARTICLE);

	//プラグイン実行
	plugin('end');

	return;
}

/* 記事表示 */
function view()
{
	global $users, $configs, $params, $pdo, $smarty;

	//記事番号取得
	if (isset($params[1])) {
		$no = $params[1];
	} else {
		$no = isset($_GET['no']) ? $_GET['no'] : 0;
	}

	if (!$no) {
		error('不正なアクセスです。');
	}

	//プラグイン実行
	plugin('begin');

	//キャッシュ設定
	if (CACHE_VIEW) {
		$smarty->caching        = 1;
		$smarty->compile_check  = false;
		$smarty->cache_dir      = CACHE_DIR;
		$smarty->cache_lifetime = CACHE_LIFETIME;
	}

	if (!$smarty->is_cached(TEMPLATE_VIEW, $no)) {
		//コメント数取得
		$stmt = $pdo->prepare('SELECT COUNT(*) FROM ' . DATABASE_TABLE_COMMENT . ' WHERE pno = :no');
		$stmt->bindValue(':no', $no, PDO::PARAM_INT);
		$flag = $stmt->execute();
		if (!$flag) {
			db_error($stmt->errorInfo());
		}

		$data    = $stmt->fetch(PDO::FETCH_NUM);
		$comment = $data[0];

		//トラックバック数取得
		$stmt = $pdo->prepare('SELECT COUNT(*) FROM ' . DATABASE_TABLE_TRACKBACK . ' WHERE pno = :no');
		$stmt->bindValue(':no', $no, PDO::PARAM_INT);
		$flag = $stmt->execute();
		if (!$flag) {
			db_error($stmt->errorInfo());
		}

		$data      = $stmt->fetch(PDO::FETCH_NUM);
		$trackback = $data[0];

		//記事取得
		$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_TABLE_ARTICLE . ' WHERE stat = 1 AND no = :no');
		$stmt->bindValue(':no', $no, PDO::PARAM_INT);
		$flag = $stmt->execute();
		if (!$flag) {
			db_error($stmt->errorInfo());
		}

		if ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
			if ($data['break']) {
				$data['text'] = '<p>' . nl2br($data['text']) . '</p>';
			}

			$smarty->assign('article', array(
				'no'        => $data['no'],
				'break'     => $data['break'],
				'comt'      => $data['comt'],
				'tb'        => $data['tb'],
				'date'      => $data['date'],
				'subj'      => $data['subj'],
				'text'      => $data['text'],
				'tag'       => create_tag($data['tag']),
				'comment'   => $comment,
				'trackback' => $trackback
			));
		} else {
			error('指定された記事は存在しません。');
		}

		//コメント取得
		$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_TABLE_COMMENT . ' WHERE pno = :no ORDER BY no');
		$stmt->bindValue(':no', $no, PDO::PARAM_INT);
		$flag = $stmt->execute();
		if (!$flag) {
			db_error($stmt->errorInfo());
		}

		$comments = array();
		while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
			$comments[] = array(
				'no'   => $data['no'],
				'pno'  => $data['pno'],
				'date' => $data['date'],
				'name' => $data['name'],
				'mail' => $data['mail'],
				'url'  => $data['url'],
				'text' => $data['text'],
				'host' => $data['host']
			);
		}
		$smarty->assign('comments', $comments);

		//トラックバック取得
		$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_TABLE_TRACKBACK . ' WHERE pno = :no ORDER BY no');
		$stmt->bindValue(':no', $no, PDO::PARAM_INT);
		$flag = $stmt->execute();
		if (!$flag) {
			db_error($stmt->errorInfo());
		}

		$trackbacks = array();
		while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
			$trackbacks[] = array(
				'no'    => $data['no'],
				'pno'   => $data['pno'],
				'date'  => $data['date'],
				'name'  => $data['name'],
				'title' => $data['title'],
				'url'   => $data['url'],
				'text'  => $data['text'],
				'host'  => $data['host']
			);
		}
		$smarty->assign('trackbacks', $trackbacks);
	}

	//プラグイン実行
	plugin('display');

	//データ表示
	output(TEMPLATE_VIEW, $no);

	//プラグイン実行
	plugin('end');

	return;
}

/* コメント投稿 */
function comment()
{
	global $users, $configs, $params, $pdo, $smarty;

	//アクセス元チェック
	if (isset($_SERVER['HTTP_REFERER']) and !preg_match('/^' . preg_quote(HTTP_PATH, '/') . '/', $_SERVER['HTTP_REFERER'])) {
		error('不正なアクセスです。');
	}

	//コメント受付チェック
	$stmt = $pdo->prepare('SELECT comt FROM ' . DATABASE_TABLE_ARTICLE . ' WHERE no = :no');
	$stmt->bindValue(':no', $_POST['no'], PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	if ($data = $stmt->fetch(PDO::FETCH_NUM)) {
		$comt = $data[0];

		if (!$comt) {
			error('この記事はコメントを受け付けていません。');
		}
	} else {
		error('指定された記事は存在しません。');
	}

	//入力内容チェック
	if ($_POST['name'] == '') {
		error('名前が入力されていません。');
	}
	if ($_POST['text'] == '') {
		error('本文が入力されていません。');
	}
	if ($_POST['url'] == 'http://') {
		$_POST['url'] = '';
	}

	if (mb_strlen($_POST['name'], 'UTF-8') > 30) {
		error('名前は30文字以内で入力してください。');
	}
	if (mb_strlen($_POST['mail'], 'UTF-8') > 80) {
		error('メールアドレスは80文字以内で入力してください。');
	}
	if (mb_strlen($_POST['url'], 'UTF-8') > 200) {
		error('URLは200文字以内で入力してください。');
	}
	if (mb_strlen($_POST['text'], 'UTF-8') > 1000) {
		error('本文は1000文字以内で入力してください。');
	}

	if (!empty($_POST['mail']) and !strpos($_POST['mail'], '@')) {
		error('メールアドレスの入力内容が正しくありません。');
	}

	//プラグイン実行
	plugin('begin');

	//Cookie登録
	if (isset($_SERVER['HTTPS']) and $_SERVER['HTTPS'] != 'off') {
		$info = parse_url(HTTPS_PATH);
	} else {
		$info = parse_url(HTTP_PATH);
	}

	if (isset($_POST['save'])) {
		setcookie('comment[name]', trim($_POST['name']), time() + COOKIE_EXPIRE, $info['path']);
		setcookie('comment[mail]', trim($_POST['mail']), time() + COOKIE_EXPIRE, $info['path']);
		setcookie('comment[url]',  trim($_POST['url']),  time() + COOKIE_EXPIRE, $info['path']);
		setcookie('comment[save]', trim($_POST['save']), time() + COOKIE_EXPIRE, $info['path']);
	} else {
		setcookie('comment[name]', '', 0, $info['path']);
		setcookie('comment[mail]', '', 0, $info['path']);
		setcookie('comment[url]',  '', 0, $info['path']);
		setcookie('comment[save]', '', 0, $info['path']);
	}

	//データ投稿
	if (($mail = trim($_POST['mail'])) == '') {
		$mail = null;
	}
	if (($url = trim($_POST['url'])) == '') {
		$url = null;
	}
	if (($host = gethostbyaddr($_SERVER['REMOTE_ADDR'])) == '') {
		$host = null;
	}
	$stmt = $pdo->prepare('INSERT INTO ' . DATABASE_TABLE_COMMENT . ' VALUES(NULL, :no, :date, :name, :mail, :url, :text, :host)');
	$stmt->bindValue(':no',   $_POST['no'], PDO::PARAM_INT);
	$stmt->bindValue(':date', date('Y-m-d H:i:s'));
	$stmt->bindValue(':name', trim($_POST['name']));
	$stmt->bindValue(':mail', $mail);
	$stmt->bindValue(':url',  $url);
	$stmt->bindValue(':text', trim($_POST['text']));
	$stmt->bindValue(':host', $host);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	//キャッシュ削除
	if (CACHE_ARTICLE or CACHE_VIEW) {
		$smarty->caching       = 1;
		$smarty->compile_check = false;
		$smarty->cache_dir     = CACHE_DIR;

		if (CACHE_ARTICLE) {
			$smarty->clear_cache(TEMPLATE_ARTICLE);
		}
		if (CACHE_VIEW) {
			$smarty->clear_cache(TEMPLATE_VIEW, intval($_POST['no']));
		}

		$smarty->caching = 0;
	}

	//プラグイン実行
	plugin('end');

	//リダイレクト
	if (REWRITE_MODE) {
		$script_url = preg_replace('/\/$/', '', HTTP_PATH);
	} else {
		$script_url = HTTP_PATH . MAIN_FILE;
	}

	header('Location: ' . $script_url . '/view/' . intval($_POST['no']) . (($users['type'] == 'mobile') ? '?' . session_name() . '=' . session_id() : ''));

	exit;
}

/* トラックバック受信 */
function trackback()
{
	global $users, $configs, $params, $pdo, $smarty;

	//トラックバック受付チェック
	$stmt = $pdo->prepare('SELECT tb FROM ' . DATABASE_TABLE_ARTICLE . ' WHERE no = :no');
	$stmt->bindValue(':no', $params[1], PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	if ($data = $stmt->fetch(PDO::FETCH_NUM)) {
		$tb = $data[0];

		if (!$tb) {
			tb_error('Forbidden Post');
		}
	} else {
		tb_error('No Article');
	}

	//受信内容チェック
	if (empty($params[1])) {
		tb_error('No TrackBack ID (tb_id)');
	}
	if (!isset($_POST['blog_name']) or $_POST['blog_name'] == '') {
		tb_error('No Name (blog_name)');
	}
	if (!isset($_POST['title']) or $_POST['title'] == '') {
		tb_error('No Title (title)');
	}
	if (!isset($_POST['url']) or $_POST['url'] == '') {
		tb_error('No URL (url)');
	}

	//プラグイン実行
	plugin('begin');

	//データ投稿
	if (($text = trim(mb_convert_encoding($_POST['excerpt'], 'UTF-8', 'auto'))) == '') {
		$text = null;
	}
	if (($host = gethostbyaddr($_SERVER['REMOTE_ADDR'])) == '') {
		$host = null;
	}
	$stmt = $pdo->prepare('INSERT INTO ' . DATABASE_TABLE_TRACKBACK . ' VALUES(NULL, :no, :date, :name, :title, :url, :text, :host)');
	$stmt->bindValue(':no',    $params[1], PDO::PARAM_INT);
	$stmt->bindValue(':date',  date('Y-m-d H:i:s'));
	$stmt->bindValue(':name',  trim(mb_convert_encoding($_POST['blog_name'], 'UTF-8', 'auto')));
	$stmt->bindValue(':title', trim(mb_convert_encoding($_POST['title'], 'UTF-8', 'auto')));
	$stmt->bindValue(':url',   trim($_POST['url']));
	$stmt->bindValue(':text',  $text);
	$stmt->bindValue(':host',  $host);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	//キャッシュ削除
	if (CACHE_ARTICLE or CACHE_VIEW) {
		$smarty->caching       = 1;
		$smarty->compile_check = false;
		$smarty->cache_dir     = CACHE_DIR;

		if (CACHE_ARTICLE) {
			$smarty->clear_cache(TEMPLATE_ARTICLE);
		}
		if (CACHE_VIEW) {
			$smarty->clear_cache(TEMPLATE_VIEW, $params[1]);
		}

		$smarty->caching = 0;
	}

	//プラグイン実行
	plugin('display');

	//データ表示
	header('Content-Type: text/xml');

	echo "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n";
	echo "<response>\n";
	echo "<error>0</error>\n";
	echo "</response>\n";

	//プラグイン実行
	plugin('end');

	return;
}

/* 過去ログ */
function archive()
{
	global $users, $configs, $params, $pdo, $smarty;

	//表示年月日取得
	if (isset($params[1])) {
		$date = $params[1];
	} else {
		$date = isset($_GET['date']) ? $_GET['date'] : '';
	}

	//プラグイン実行
	plugin('begin');

	if ($date) {
		//コメント数取得
		$stmt = $pdo->query('SELECT pno, COUNT(*) FROM ' . DATABASE_TABLE_COMMENT . ' GROUP BY pno');
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}

		$comments = array();
		while ($data = $stmt->fetch(PDO::FETCH_NUM)) {
			$comments[$data[0]] = $data[1];
		}

		//トラックバック数取得
		$stmt = $pdo->query('SELECT pno, COUNT(*) FROM ' . DATABASE_TABLE_TRACKBACK . ' GROUP BY pno');
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}

		$trackbacks = array();
		while ($data = $stmt->fetch(PDO::FETCH_NUM)) {
			$trackbacks[$data[0]] = $data[1];
		}

		//記事取得
		if (preg_match('/^(\d\d\d\d)$/', $date, $matches)) {
			$smarty->assign('view', array(
				'flag' => true,
				'year' => $matches[1]
			));

			if (DATABASE_TYPE == 'mysql') {
				$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_TABLE_ARTICLE . ' WHERE stat = 1 AND DATE_FORMAT(date, \'%Y\') = :date ORDER BY date DESC');
			} else {
				$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_TABLE_ARTICLE . ' WHERE stat = 1 AND STRFTIME(\'%Y\', date) = :date ORDER BY date DESC');
			}
		} elseif (preg_match('/^(\d\d\d\d)(\d\d)$/', $date, $matches)) {
			$smarty->assign('view', array(
				'flag'  => true,
				'year'  => $matches[1],
				'month' => $matches[2]
			));

			if (DATABASE_TYPE == 'mysql') {
				$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_TABLE_ARTICLE . ' WHERE stat = 1 AND DATE_FORMAT(date, \'%Y%m\') = :date ORDER BY date DESC');
			} else {
				$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_TABLE_ARTICLE . ' WHERE stat = 1 AND STRFTIME(\'%Y%m\', date) = :date ORDER BY date DESC');
			}
		} elseif (preg_match('/^(\d\d\d\d)(\d\d)(\d\d)$/', $date, $matches)) {
			$smarty->assign('view', array(
				'flag'  => true,
				'year'  => $matches[1],
				'month' => $matches[2],
				'day'   => $matches[3]
			));

			if (DATABASE_TYPE == 'mysql') {
				$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_TABLE_ARTICLE . ' WHERE stat = 1 AND DATE_FORMAT(date, \'%Y%m%d\') = :date ORDER BY date DESC');
			} else {
				$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_TABLE_ARTICLE . ' WHERE stat = 1 AND STRFTIME(\'%Y%m%d\', date) = :date ORDER BY date DESC');
			}
		} else {
			error('不正なアクセスです。');
		}
		$stmt->bindValue(':date', $date);
		$flag = $stmt->execute();
		if (!$flag) {
			db_error($stmt->errorInfo());
		}

		$articles = array();
		while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
			if ($data['break']) {
				$data['text'] = '<p>' . nl2br($data['text']) . '</p>';
			}

			$articles[] = array(
				'no'        => $data['no'],
				'break'     => $data['break'],
				'comt'      => $data['comt'],
				'tb'        => $data['tb'],
				'date'      => $data['date'],
				'subj'      => $data['subj'],
				'text'      => $data['text'],
				'tag'       => create_tag($data['tag']),
				'comment'   => isset($comments[$data['no']])   ? $comments[$data['no']]   : 0,
				'trackback' => isset($trackbacks[$data['no']]) ? $trackbacks[$data['no']] : 0
			);
		}
		$smarty->assign('articles', $articles);
	} else {
		//月一覧取得
		if (DATABASE_TYPE == 'mysql') {
			$stmt = $pdo->query('SELECT DATE_FORMAT(date, \'%Y-%m\') AS month FROM ' . DATABASE_TABLE_ARTICLE . ' WHERE stat = 1 GROUP BY month ORDER BY month DESC');
		} else {
			$stmt = $pdo->query('SELECT STRFTIME(\'%Y-%m\', date) AS month FROM ' . DATABASE_TABLE_ARTICLE . ' WHERE stat = 1 GROUP BY month ORDER BY month DESC');
		}
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}

		$dates = array();
		while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
			if (preg_match('/^(\d\d\d\d)\-(\d\d)$/', $data['month'], $matches)) {
				$year   = $matches[1];
				$month  = $matches[2];
			} else {
				exit('投稿日時の形式が不正です。');
			}

			$dates[] = array(
				'year'  => $year,
				'month' => $month,
			);
		}
		$smarty->assign('dates', $dates);
	}

	//プラグイン実行
	plugin('display');

	//データ表示
	output(TEMPLATE_ARCHIVE);

	//プラグイン実行
	plugin('end');

	return;
}

/* ログ検索 */
function search()
{
	global $users, $configs, $params, $pdo, $smarty;

	//検索対象取得
	if (isset($params[1])) {
		$target = $params[1];
	} else {
		$target = isset($_GET['target']) ? $_GET['target'] : '';
	}
	$smarty->assign('target', $target);

	//検索語句取得
	if (isset($params[2])) {
		$word = preg_replace('/\+/', ' ', $params[2]);
	} else {
		$word = isset($_GET['word']) ? $_GET['word'] : '';
	}
	$smarty->assign('word', $word);

	//プラグイン実行
	plugin('begin');

	if ($word) {
		//コメント数取得
		$stmt = $pdo->query('SELECT pno, COUNT(*) FROM ' . DATABASE_TABLE_COMMENT . ' GROUP BY pno');
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}

		$comments = array();
		while ($data = $stmt->fetch(PDO::FETCH_NUM)) {
			$comments[$data[0]] = $data[1];
		}

		//トラックバック数取得
		$stmt = $pdo->query('SELECT pno, COUNT(*) FROM ' . DATABASE_TABLE_TRACKBACK . ' GROUP BY pno');
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}

		$trackbacks = array();
		while ($data = $stmt->fetch(PDO::FETCH_NUM)) {
			$trackbacks[$data[0]] = $data[1];
		}

		//検索結果取得
		if ($target == 'tag') {
			$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_TABLE_ARTICLE . ' WHERE stat = 1 AND (tag = :word1 OR tag LIKE :word2 OR tag LIKE :word3 OR tag LIKE :word4) ORDER BY date DESC');
			$stmt->bindValue(':word1', $word);
			$stmt->bindValue(':word2', $word . ',%');
			$stmt->bindValue(':word3', '%,' . $word);
			$stmt->bindValue(':word4', '%,' . $word . ',%');
		} else {
			$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_TABLE_ARTICLE . ' WHERE stat = 1 AND (subj LIKE :word1 OR text LIKE :word2 OR tag LIKE :word3) ORDER BY date DESC');
			$stmt->bindValue(':word1', '%' . $word . '%');
			$stmt->bindValue(':word2', '%' . $word . '%');
			$stmt->bindValue(':word3', '%' . $word . '%');
		}
		$flag = $stmt->execute();
		if (!$flag) {
			db_error($stmt->errorInfo());
		}

		$articles = array();
		while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
			if ($data['break']) {
				$data['text'] = '<p>' . nl2br($data['text']) . '</p>';
			}

			$articles[] = array(
				'no'        => $data['no'],
				'break'     => $data['break'],
				'comt'      => $data['comt'],
				'tb'        => $data['tb'],
				'date'      => $data['date'],
				'subj'      => $data['subj'],
				'text'      => $data['text'],
				'tag'       => create_tag($data['tag']),
				'comment'   => isset($comments[$data['no']])   ? $comments[$data['no']]   : 0,
				'trackback' => isset($trackbacks[$data['no']]) ? $trackbacks[$data['no']] : 0
			);
		}
		$smarty->assign('articles', $articles);
	}

	//プラグイン実行
	plugin('display');

	//データ表示
	output(TEMPLATE_SEARCH);

	//プラグイン実行
	plugin('end');

	return;
}

/* RSS配信 */
function rss()
{
	global $users, $configs, $params, $pdo, $smarty;

	//プラグイン実行
	plugin('begin');

	//キャッシュ設定
	if (CACHE_RSS) {
		$smarty->caching        = 1;
		$smarty->compile_check  = false;
		$smarty->cache_dir      = CACHE_DIR;
		$smarty->cache_lifetime = CACHE_LIFETIME;
	}

	if (!$smarty->is_cached(TEMPLATE_RSS)) {
		//記事取得
		$stmt = $pdo->query('SELECT * FROM ' . DATABASE_TABLE_ARTICLE . ' WHERE stat = 1 ORDER BY date DESC LIMIT ' . $configs['_blog']['rss_size']);
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}

		$articles = array();
		while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
			if ($data['break']) {
				$data['text'] = '<p>' . nl2br($data['text']) . '</p>';
			}

			$description = strip_tags($data['text']);
			$description = strlen($description) > 255 ? mb_strimwidth($description, 0, 252, '...', 'UTF-8') : $description;

			$articles[] = array(
				'no'          => $data['no'],
				'break'       => $data['break'],
				'comt'        => $data['comt'],
				'tb'          => $data['tb'],
				'date'        => $data['date'],
				'subj'        => $data['subj'],
				'text'        => $data['text'],
				'description' => $description,
				'tag'         => $data['tag']
			);
		}
		$smarty->assign('articles', $articles);

		//最終更新日時取得
		$stmt = $pdo->query('SELECT * FROM ' . DATABASE_TABLE_ARTICLE . ' WHERE stat = 1 ORDER BY date DESC LIMIT 1');
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}

		if ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
			$smarty->assign('last', array(
				'date' => $data['date']
			));
		}
	}

	//プラグイン実行
	plugin('display');

	//データ表示
	output(TEMPLATE_RSS, null, 'text/xml');

	//プラグイン実行
	plugin('end');

	return;
}

/* ログイン状態チェック */
function isloggedin()
{
	global $users, $configs, $params, $pdo, $smarty;

	if (isset($_SERVER['HTTPS']) and $_SERVER['HTTPS'] != 'off') {
		$info = parse_url(HTTPS_PATH);
	} else {
		$info = parse_url(HTTP_PATH);
	}

	if ($users['serial'] and ADMIN_SERIAL) {
		//固体識別情報確認
		if (in_array($users['serial'], explode(',', ADMIN_SERIAL))) {
			return true;
		}
	} elseif (isset($_POST['pwd'])) {
		//パスワード認証
		if ($_POST['pwd'] == ADMIN_PWD) {
			$_SESSION['login'] = true;

			if (isset($_POST['session'])) {
				$session = md5(uniqid(rand(), true));

				if (touch(LOGIN_DIR . $session)) {
					chmod(LOGIN_DIR . $session, 0644);
				} else {
					exit('ログイン情報を保持できません。');
				}

				setcookie('session', $session, time() + COOKIE_EXPIRE, $info['path']);
			}
		}

		header('Location: ' . (HTTPS_PATH ? HTTPS_PATH : HTTP_PATH) . MAIN_FILE . '?mode=admin&work=default' . (($users['type'] == 'mobile') ? '&' . session_name() . '=' . session_id() : ''));

		exit;
	} elseif (empty($_SESSION['login'])) {
		//ログイン状態を復元
		if (isset($_COOKIE['session'])) {
			if ($dir = scandir(LOGIN_DIR)) {
				foreach ($dir as $entry) {
					if (is_file(LOGIN_DIR . $entry) and time() - filemtime(LOGIN_DIR . $entry) > COOKIE_EXPIRE) {
						unlink(LOGIN_DIR . $entry);
					}
				}
			} else {
				exit('ログイン情報格納ディレクトリを開けません。');
			}

			if (file_exists(LOGIN_DIR . $_COOKIE['session'])) {
				$_SESSION['login'] = true;

				$session = md5(uniqid(rand(), true));

				if (rename(LOGIN_DIR . $_COOKIE['session'], LOGIN_DIR . $session) and touch(LOGIN_DIR . $session)) {
					setcookie('session', $session, time() + COOKIE_EXPIRE, $info['path']);
				} else {
					exit('ログイン情報を保持できません。');
				}
			}
		}
	}

	if (isset($_SESSION['login']) and $_SESSION['login'] == true) {
		return true;
	} else {
		return false;
	}
}

/* 管理者用 - 認証画面 */
function admin()
{
	global $users, $configs, $params, $pdo, $smarty;

	//プラグイン実行
	plugin('begin');

	//エラーメッセージ取得
	if ($_REQUEST['work'] == 'default') {
		$smarty->append('errors', 'パスワードが違います。');
	}

	//プラグイン実行
	plugin('display');

	//データ表示
	output(TEMPLATE_ADMIN);

	//プラグイン実行
	plugin('end');

	return;
}

/* 管理者用 - 投稿画面 */
function admin_form()
{
	global $users, $configs, $params, $pdo, $smarty;

	//ワンタイムトークン作成
	$smarty->assign('token', create_token());

	//プラグイン実行
	plugin('begin');

	//フォームデータ取得
	if (isset($_POST['work']) and $_POST['work'] == 'regist') {
		$form_no    = $_POST['no'];
		$form_stat  = $_POST['stat'];
		$form_break = $_POST['break'];
		$form_comt  = $_POST['comt'];
		$form_tb    = $_POST['tb'];
		$form_date  = $_POST['date'];
		$form_subj  = $_POST['subj'];
		$form_text  = $_POST['text'];
		$form_tag   = $_POST['tag'];
	} elseif (isset($_GET['no'])) {
		$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_TABLE_ARTICLE . ' WHERE no = :no');
		$stmt->bindValue(':no', $_GET['no'], PDO::PARAM_INT);
		$flag = $stmt->execute();
		if (!$flag) {
			db_error($stmt->errorInfo());
		}

		if ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
			$form_no    = $data['no'];
			$form_stat  = $data['stat'];
			$form_break = $data['break'];
			$form_comt  = $data['comt'];
			$form_tb    = $data['tb'];
			$form_date  = $data['date'];
			$form_subj  = $data['subj'];
			$form_text  = $data['text'];
			$form_tag   = $data['tag'];
		} else {
			error('指定された記事は存在しません。');
		}
	} else {
		$form_no    = '';
		$form_stat  = $configs['_blog']['default_stat']  ? true : false;
		$form_break = $configs['_blog']['default_break'] ? true : false;
		$form_comt  = $configs['_blog']['default_comt']  ? true : false;
		$form_tb    = $configs['_blog']['default_tb']    ? true : false;
		$form_date  = date('Y-m-d H:i:00');
		$form_subj  = '';
		$form_text  = '';
		$form_tag   = '';
	}

	$smarty->assign('form', array(
		'no'    => $form_no,
		'stat'  => $form_stat,
		'break' => $form_break,
		'comt'  => $form_comt,
		'tb'    => $form_tb,
		'date'  => $form_date,
		'subj'  => $form_subj,
		'text'  => $form_text,
		'tag'   => $form_tag
	));

	//アップロードファイル取得
	if ($dir = scandir(FILE_DIR)) {
		$files = array();
		foreach ($dir as $entry) {
			if ($entry == '.' or $entry == '..') {
				continue;
			}

			if (preg_match('/^[\w\-]+\.(gif|jpeg|jpg|jpe|png)$/i', $entry)) {
				list($file_width, $file_height) = getimagesize(FILE_DIR . $entry);
				$value = '<img src="' . HTTP_PATH . FILE_DIR . $entry . '" alt="' . $entry . '" width="' . $file_width . '" height="' . $file_height . '" />';
			} else {
				$value = '<a href="' . HTTP_PATH . FILE_DIR . $entry . '">' . $entry . '</a>';
			}

			$files[] = array(
				'name'  => $entry,
				'value' => $value
			);
		}

		$smarty->assign('files', $files);
	} else {
		exit('アップロードファイル格納ディレクトリを開けません。');
	}

	//プラグイン実行
	plugin('display');

	//データ表示
	output(TEMPLATE_ADMIN_FORM);

	//プラグイン実行
	plugin('end');

	return;
}

/* 管理者用 - 記事投稿 */
function admin_regist()
{
	global $users, $configs, $params, $pdo, $smarty;

	//アクセス元チェック
	if (isset($_SERVER['HTTP_REFERER']) and !preg_match('/^' . preg_quote(HTTPS_PATH ? HTTPS_PATH : HTTP_PATH, '/') . '/', $_SERVER['HTTP_REFERER'])) {
		error('不正なアクセスです。');
	}

	//ワンタイムトークン比較
	if (!isset($_POST['preview']) and !check_token()) {
		error('不正なアクセスです。');
	}

	//入力内容チェック
	if (!isset($_POST['stat'])) {
		$_POST['stat'] = 0;
	}
	if (!isset($_POST['break'])) {
		$_POST['break'] = 0;
	}
	if (!isset($_POST['comt'])) {
		$_POST['comt'] = 0;
	}
	if (!isset($_POST['tb'])) {
		$_POST['tb'] = 0;
	}
	if ($_POST['tag'] != '') {
		$_POST['tag'] = preg_replace('/\s*,\s*/', ',', $_POST['tag']);
	}

	if ($_POST['subj'] == '') {
		$smarty->append('errors', '題名が入力されていません。');
	}
	if ($_POST['text'] == '') {
		$smarty->append('errors', '本文が入力されていません。');
	}

	if (mb_strlen($_POST['subj'], 'UTF-8') > 30) {
		$smarty->append('errors', '題名は30文字以内で入力してください。');
	}
	if (mb_strlen($_POST['text'], 'UTF-8') > 3000) {
		$smarty->append('errors', '本文は3000文字以内で入力してください。');
	}
	if (mb_strlen($_POST['tag'], 'UTF-8') > 200) {
		$smarty->append('errors', 'タグは200文字以内で入力してください。');
	}

	if (is_array($_POST['date'])) {
		$year   = mb_convert_kana($_POST['date']['year'], 'n', 'UTF-8');
		$month  = mb_convert_kana($_POST['date']['month'], 'n', 'UTF-8');
		$day    = mb_convert_kana($_POST['date']['day'], 'n', 'UTF-8');
		$hour   = mb_convert_kana($_POST['date']['hour'], 'n', 'UTF-8');
		$minute = mb_convert_kana($_POST['date']['minute'], 'n', 'UTF-8');

		$_POST['date'] = sprintf('%04d-%02d-%02d %02d:%02d:00', $year, $month, $day, $hour, $minute);
	} else {
		if (!preg_match('/^\d\d\d\d-\d\d-\d\d \d\d:\d\d$/', $_POST['date'])) {
			$smarty->append('errors', '日時が正しく指定されていません。');
		}

		$_POST['date'] = $_POST['date'] . ':00';
	}

	if (isset($_POST['preview']) or $smarty->get_template_vars('errors')) {
		if (isset($_POST['preview']) and !$smarty->get_template_vars('errors') and check_token()) {
			$smarty->assign('article', array(
				'preview' => true,
				'no'      => $_POST['no'] ? $_POST['no'] : 0,
				'stat'    => $_POST['stat'],
				'break'   => $_POST['break'],
				'comt'    => $_POST['comt'],
				'tb'      => $_POST['tb'],
				'date'    => $_POST['date'],
				'subj'    => trim($_POST['subj']),
				'text'    => $_POST['break'] ? '<p>' . nl2br(trim($_POST['text'])) . '</p>' : trim($_POST['text']),
				'tag'     => trim($_POST['tag'])
			));
		}

		admin_form();

		exit;
	}

	//プラグイン実行
	plugin('begin');

	//データ投稿
	if (($tag = trim($_POST['tag'])) == '') {
		$tag = null;
	}
	if (empty($_POST['no'])) {
		$stmt = $pdo->prepare('INSERT INTO ' . DATABASE_TABLE_ARTICLE . ' VALUES(NULL, :stat, :break, :comt, :tb, :date, :subj, :text, :tag)');
		$stmt->bindValue(':stat',  $_POST['stat'], PDO::PARAM_INT);
		$stmt->bindValue(':break', $_POST['break'], PDO::PARAM_INT);
		$stmt->bindValue(':comt',  $_POST['comt'], PDO::PARAM_INT);
		$stmt->bindValue(':tb',    $_POST['tb'], PDO::PARAM_INT);
		$stmt->bindValue(':date',  $_POST['date']);
		$stmt->bindValue(':subj',  trim($_POST['subj']));
		$stmt->bindValue(':text',  trim($_POST['text']));
		$stmt->bindValue(':tag',   $tag);
		$flag = $stmt->execute();
		if (!$flag) {
			db_error($stmt->errorInfo());
		}

		$smarty->assign('message', '記事を新規に投稿しました。');
	} else {
		$stmt = $pdo->prepare('UPDATE ' . DATABASE_TABLE_ARTICLE . ' SET stat = :stat, break = :break, comt = :comt, tb = :tb, date = :date, subj = :subj, text = :text, tag = :tag WHERE no = :no');
		$stmt->bindValue(':stat',  $_POST['stat'], PDO::PARAM_INT);
		$stmt->bindValue(':break', $_POST['break'], PDO::PARAM_INT);
		$stmt->bindValue(':comt',  $_POST['comt'], PDO::PARAM_INT);
		$stmt->bindValue(':tb',    $_POST['tb'], PDO::PARAM_INT);
		$stmt->bindValue(':date',  $_POST['date']);
		$stmt->bindValue(':subj',  trim($_POST['subj']));
		$stmt->bindValue(':text',  trim($_POST['text']));
		$stmt->bindValue(':tag',   $tag);
		$stmt->bindValue(':no',    $_POST['no'], PDO::PARAM_INT);
		$flag = $stmt->execute();
		if (!$flag) {
			db_error($stmt->errorInfo());
		}

		$smarty->assign('message', 'No.' . $_POST['no'] . 'の記事を編集しました。');
	}

	//トラックバック送信
	if (!empty($_POST['tb_url'])) {
		if (empty($_POST['no'])) {
			$no = $pdo->lastInsertId();
		} else {
			$no = $_POST['no'];
		}

		if (REWRITE_MODE) {
			$url = preg_replace('/\/$/', '', HTTP_PATH);
		} else {
			$url = HTTP_PATH . MAIN_FILE;
		}

		$excerpt = str_replace("\n", '', strip_tags($_POST['text']));
		$excerpt = strlen($excerpt) > 255 ? mb_strimwidth($excerpt, 0, 252, '...', 'UTF-8') : $excerpt;
		$query   = 'title=' . urlencode($_POST['subj']) . '&url=' . urlencode($url . '/view/' . $no) . '&excerpt=' . urlencode($excerpt) . '&blog_name=' . urlencode($configs['_blog']['title']);

		$errors = array();

		foreach (explode("\n", $_POST['tb_url']) as $tb_url) {
			$info = parse_url($tb_url);

			$request  = "POST " . $info['path'] . (isset($info['query']) ? '?' . $info['query'] : '') . " HTTP/1.0\r\n";
			$request .= "Host: " . $info['host'] . "\r\n";
			$request .= "User-Agent: PHP Blog\r\n";
			$request .= "Content-Type: application/x-www-form-urlencoded\r\n";
			$request .= "Content-Length: " . strlen($query) . "\r\n";
			$request .= "\r\n";
			$request .= "$query\r\n";

			$data = '';

			if ($sock = fsockopen($info['host'], 80)) {
				fputs($sock, $request);
				while (!feof($sock)) {
					$data .= fgets($sock);
				}
				fclose($sock);
			} else {
				$errors[] = 'Socket error : ' . $tb_url;
			}

			if (!preg_match('/<error>0<\/error>/', $data, $matches)) {
				$errors[] = 'Response error from ' . $tb_url;
			} elseif (preg_match('/<error>1<\/error>[^<]*<message>(.+)<\/message>/', $data, $matches)) {
				$errors[] = 'Response error : ' . $matches[1] . ' from ' . $tb_url;
			}
		}

		if (!empty($errors)) {
			$smarty->assign('message', $smarty->get_template_vars('message') . 'トラックバックの送信に失敗しました。' . implode('、', $errors));
		}
	}

	//ワンタイムトークン破棄
	destroy_token();

	//キャッシュ削除
	if (CACHE_ARTICLE or CACHE_VIEW or CACHE_RSS) {
		$smarty->caching       = 1;
		$smarty->compile_check = false;
		$smarty->cache_dir     = CACHE_DIR;

		if (CACHE_ARTICLE) {
			$smarty->clear_cache(TEMPLATE_ARTICLE);
		}
		if (CACHE_VIEW and !empty($_POST['no'])) {
			$smarty->clear_cache(TEMPLATE_VIEW, $_POST['no']);
		}
		if (CACHE_RSS) {
			$smarty->clear_cache(TEMPLATE_RSS);
		}

		$smarty->caching = 0;
	}

	//プラグイン実行
	plugin('end');

	//記事一覧
	admin_edit();

	return;
}

/* 管理者用 - 記事一覧 */
function admin_edit()
{
	global $users, $configs, $params, $pdo, $smarty;

	//ワンタイムトークン作成
	$smarty->assign('token', create_token());

	//表示ページ取得
	$page = isset($_GET['page']) ? $_GET['page'] : 0;

	//プラグイン実行
	plugin('begin');

	//記事取得
	$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_TABLE_ARTICLE . ' ORDER BY date DESC LIMIT :start, :limit');
	$stmt->bindValue(':start', intval($configs['_blog']['admin_size']) * $page, PDO::PARAM_INT);
	$stmt->bindValue(':limit', intval($configs['_blog']['admin_size']), PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	$articles = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		if ($data['break']) {
			$data['text'] = '<p>' . nl2br($data['text']) . '</p>';
		}

		$description = strip_tags($data['text']);
		$description = strlen($description) > 255 ? mb_strimwidth($description, 0, 252, '...', 'UTF-8') : $description;

		$articles[] = array(
			'no'          => $data['no'],
			'stat'        => $data['stat'],
			'break'       => $data['break'],
			'comt'        => $data['comt'],
			'tb'          => $data['tb'],
			'date'        => $data['date'],
			'subj'        => $data['subj'],
			'text'        => $data['text'],
			'description' => $description,
			'tag'         => create_tag($data['tag'])
		);
	}
	$smarty->assign('articles', $articles);

	//記事数取得
	$stmt = $pdo->query('SELECT COUNT(*) FROM ' . DATABASE_TABLE_ARTICLE);
	if (!$stmt) {
		db_error($pdo->errorInfo());
	}

	$data  = $stmt->fetch(PDO::FETCH_NUM);
	$count = $data[0];

	$smarty->assign('count', $count);

	//ページ前後移動
	if ($page > 0) {
		$smarty->assign('prev', array(
			'id'   => $page - 1,
			'flag' => true
		));
	}
	if (intval(($count - 1) / $configs['_blog']['admin_size']) > $page) {
		$smarty->assign('next', array(
			'id'   => $page + 1,
			'flag' => true
		));
	}

	//ページ直接移動
	$pages = array();
	for ($i = 0; $i <= intval(($count - 1) / $configs['_blog']['admin_size']); $i++) {
		$pages[] = array(
			'id'   => $i,
			'flag' => $page == $i ? true : false
		);
	}
	$smarty->assign('pages', $pages);

	//プラグイン実行
	plugin('display');

	//データ表示
	output(TEMPLATE_ADMIN_EDIT);

	//プラグイン実行
	plugin('end');

	return;
}

/* 管理者用 - 記事削除 */
function admin_del()
{
	global $users, $configs, $params, $pdo, $smarty;

	//アクセス元チェック
	if (isset($_SERVER['HTTP_REFERER']) and !preg_match('/^' . preg_quote(HTTPS_PATH ? HTTPS_PATH : HTTP_PATH, '/') . '/', $_SERVER['HTTP_REFERER'])) {
		error('不正なアクセスです。');
	}

	//ワンタイムトークン比較
	if (!check_token()) {
		error('不正なアクセスです。');
	}

	//入力内容チェック
	if (empty($_REQUEST['no'])) {
		error('削除したい記事を選択してください。');
	}

	//プラグイン実行
	plugin('begin');

	//データ削除
	$stmt = $pdo->prepare('DELETE FROM ' . DATABASE_TABLE_ARTICLE . ' WHERE no = :no');
	$stmt->bindValue(':no', $_REQUEST['no'], PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	$stmt = $pdo->prepare('DELETE FROM ' . DATABASE_TABLE_COMMENT . ' WHERE pno = :no');
	$stmt->bindValue(':no', $_REQUEST['no'], PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	$stmt = $pdo->prepare('DELETE FROM ' . DATABASE_TABLE_TRACKBACK . ' WHERE pno = :no');
	$stmt->bindValue(':no', $_REQUEST['no'], PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	if (DATABASE_TYPE == 'mysql') {
		$stmt = $pdo->query('ALTER TABLE ' . DATABASE_TABLE_ARTICLE . ' AUTO_INCREMENT = 0');
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}

		$stmt = $pdo->query('ALTER TABLE ' . DATABASE_TABLE_COMMENT . ' AUTO_INCREMENT = 0');
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}

		$stmt = $pdo->query('ALTER TABLE ' . DATABASE_TABLE_TRACKBACK . ' AUTO_INCREMENT = 0');
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}
	}

	$smarty->assign('message', 'No.' . $_REQUEST['no'] . 'の記事を削除しました。');

	//ワンタイムトークン破棄
	destroy_token();

	//キャッシュ削除
	if (CACHE_ARTICLE or CACHE_VIEW or CACHE_RSS) {
		$smarty->caching       = 1;
		$smarty->compile_check = false;
		$smarty->cache_dir     = CACHE_DIR;

		if (CACHE_ARTICLE) {
			$smarty->clear_cache(TEMPLATE_ARTICLE);
		}
		if (CACHE_VIEW) {
			$smarty->clear_cache(TEMPLATE_VIEW, $_REQUEST['no']);
		}
		if (CACHE_RSS) {
			$smarty->clear_cache(TEMPLATE_RSS);
		}

		$smarty->caching = 0;
	}

	//プラグイン実行
	plugin('end');

	//記事一覧
	admin_edit();

	return;
}

/* 管理者用 - ファイル一覧 */
function admin_file()
{
	global $users, $configs, $params, $pdo, $smarty;

	//ワンタイムトークン作成
	$smarty->assign('token', create_token());

	//プラグイン実行
	plugin('begin');

	//アップロードファイル一覧取得
	if ($dir = scandir(FILE_DIR)) {
		$files = array();
		foreach ($dir as $entry) {
			if (!is_file(FILE_DIR . $entry)) {
				continue;
			}

			$size = intval(filesize(FILE_DIR . $entry) / 1024);
			if ($size == 0) {
				$size = 1;
			}

			$files[] = array(
				'name' => $entry,
				'size' => $size,
				'date' => date('Y-m-d H:i:s', filemtime(FILE_DIR . $entry))
			);
		}

		$smarty->assign('files', $files);
	} else {
		exit('アップロードファイル格納ディレクトリを開けません。');
	}

	//プラグイン実行
	plugin('display');

	//データ表示
	output(TEMPLATE_ADMIN_FILE);

	//プラグイン実行
	plugin('end');

	return;
}

/* 管理者用 - ファイル追加 */
function admin_file_add()
{
	global $users, $configs, $params, $pdo, $smarty;

	//アクセス元チェック
	if (isset($_SERVER['HTTP_REFERER']) and !preg_match('/^' . preg_quote(HTTPS_PATH ? HTTPS_PATH : HTTP_PATH, '/') . '/', $_SERVER['HTTP_REFERER'])) {
		error('不正なアクセスです。');
	}

	//ワンタイムトークン比較
	if (!check_token()) {
		error('不正なアクセスです。');
	}

	//プラグイン実行
	plugin('begin');

	//ファイルアップロード
	if (is_uploaded_file($_FILES['file']['tmp_name'])) {
		if (!preg_match('/^[\w\.\~\-\&\#\+\=\;\@\%]+$/', $_FILES['file']['name'])) {
			error('ファイル名は半角英数字で指定してください。');
		}
		if (file_exists(FILE_DIR . $_FILES['file']['name'])) {
			error('ファイル「' . $_FILES['file']['name'] . '」はすでに存在します。');
		}

		if (move_uploaded_file($_FILES['file']['tmp_name'], FILE_DIR . $_FILES['file']['name'])) {
			chmod(FILE_DIR . $_FILES['file']['name'], 0644);
		} else {
			exit('ファイルをアップロードできません。');
		}
	} else {
		error('ファイルを選択してください。');
	}

	//ワンタイムトークン破棄
	destroy_token();

	//プラグイン実行
	plugin('end');

	//ファイル一覧
	admin_file();

	return;
}

/* 管理者用 - ファイル削除 */
function admin_file_del()
{
	global $users, $configs, $params, $pdo, $smarty;

	//アクセス元チェック
	if (isset($_SERVER['HTTP_REFERER']) and !preg_match('/^' . preg_quote(HTTPS_PATH ? HTTPS_PATH : HTTP_PATH, '/') . '/', $_SERVER['HTTP_REFERER'])) {
		error('不正なアクセスです。');
	}

	//ワンタイムトークン比較
	if (!check_token()) {
		error('不正なアクセスです。');
	}

	//入力内容チェック
	if (!isset($_POST['name'])) {
		error('削除したいファイルを選択してください。');
	}

	//プラグイン実行
	plugin('begin');

	//ファイル削除
	foreach ($_POST['name'] as $name) {
		if (preg_match('/^[\w\-]+\.\w+$/', $name)) {
			unlink(FILE_DIR . $name);
		}
	}

	//ワンタイムトークン破棄
	destroy_token();

	//プラグイン実行
	plugin('end');

	//ファイル一覧
	admin_file();

	return;
}

/* 管理者用 - コメント一覧 */
function admin_comment()
{
	global $users, $configs, $params, $pdo, $smarty;

	//ワンタイムトークン作成
	$smarty->assign('token', create_token());

	//表示ページ取得
	$page = isset($_GET['page']) ? $_GET['page'] : 0;

	//プラグイン実行
	plugin('begin');

	//コメント取得
	$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_TABLE_COMMENT . ' ORDER BY no DESC LIMIT :start, :limit');
	$stmt->bindValue(':start', intval($configs['_blog']['admin_size']) * $page, PDO::PARAM_INT);
	$stmt->bindValue(':limit', intval($configs['_blog']['admin_size']), PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	$comments = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$description = strip_tags($data['text']);
		$description = strlen($description) > 255 ? mb_strimwidth($description, 0, 252, '...', 'UTF-8') : $description;

		$comments[] = array(
			'no'          => $data['no'],
			'pno'         => $data['pno'],
			'date'        => $data['date'],
			'name'        => $data['name'],
			'mail'        => $data['mail'],
			'url'         => $data['url'],
			'text'        => $data['text'],
			'description' => $description,
			'host'        => $data['host']
		);
	}
	$smarty->assign('comments', $comments);

	//コメント数取得
	$stmt = $pdo->query('SELECT COUNT(*) FROM ' . DATABASE_TABLE_COMMENT);
	if (!$stmt) {
		db_error($pdo->errorInfo());
	}

	$data  = $stmt->fetch(PDO::FETCH_NUM);
	$count = $data[0];

	$smarty->assign('count', $count);

	//ページ前後移動
	if ($page > 0) {
		$smarty->assign('prev', array(
			'id'   => $page - 1,
			'flag' => true
		));
	}
	if (intval(($count - 1) / $configs['_blog']['admin_size']) > $page) {
		$smarty->assign('next', array(
			'id'   => $page + 1,
			'flag' => true
		));
	}

	//ページ直接移動
	$pages = array();
	for ($i = 0; $i <= intval(($count - 1) / $configs['_blog']['admin_size']); $i++) {
		$pages[] = array(
			'id'   => $i,
			'flag' => $page == $i ? true : false
		);
	}
	$smarty->assign('pages', $pages);

	//プラグイン実行
	plugin('display');

	//データ表示
	output(TEMPLATE_ADMIN_COMMENT);

	//プラグイン実行
	plugin('end');

	return;
}

/* 管理者用 - コメント削除 */
function admin_comment_del()
{
	global $users, $configs, $params, $pdo, $smarty;

	//アクセス元チェック
	if (isset($_SERVER['HTTP_REFERER']) and !preg_match('/^' . preg_quote(HTTPS_PATH ? HTTPS_PATH : HTTP_PATH, '/') . '/', $_SERVER['HTTP_REFERER'])) {
		error('不正なアクセスです。');
	}

	//ワンタイムトークン比較
	if (!check_token()) {
		error('不正なアクセスです。');
	}

	//入力内容チェック
	if (empty($_POST['no'])) {
		error('削除したいコメントを選択してください。');
	}

	$dels = array();
	foreach ($_POST['no'] as $no) {
		$dels[] = intval($no);
	}

	//プラグイン実行
	plugin('begin');

	//データ削除
	$stmt = $pdo->query('DELETE FROM ' . DATABASE_TABLE_COMMENT . ' WHERE no IN(' . implode(',', $dels) . ')');
	if (!$stmt) {
		db_error($pdo->errorInfo());
	}

	if (DATABASE_TYPE == 'mysql') {
		$stmt = $pdo->query('ALTER TABLE ' . DATABASE_TABLE_COMMENT . ' AUTO_INCREMENT = 0');
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}
	}

	$smarty->assign('message', 'No.' . implode('、', $dels) . 'のコメントを削除しました。');

	//ワンタイムトークン破棄
	destroy_token();

	//キャッシュ削除
	if (CACHE_ARTICLE or CACHE_VIEW) {
		$smarty->caching       = 1;
		$smarty->compile_check = false;
		$smarty->cache_dir     = CACHE_DIR;

		if (CACHE_ARTICLE) {
			$smarty->clear_cache(TEMPLATE_ARTICLE);
		}
		if (CACHE_VIEW) {
			$stmt = $pdo->query('SELECT pno FROM ' . DATABASE_TABLE_COMMENT . ' WHERE no IN(' . implode(',', $dels) . ') GROUP BY pno');
			if (!$stmt) {
				db_error($pdo->errorInfo());
			}

			while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
				$smarty->clear_cache(TEMPLATE_VIEW, $data['pno']);
			}
		}

		$smarty->caching = 0;
	}

	//プラグイン実行
	plugin('end');

	//コメント一覧
	admin_comment();

	return;
}

/* 管理者用 - トラックバック一覧 */
function admin_trackback()
{
	global $users, $configs, $params, $pdo, $smarty;

	//ワンタイムトークン作成
	$smarty->assign('token', create_token());

	//表示ページ取得
	$page = isset($_GET['page']) ? $_GET['page'] : 0;

	//プラグイン実行
	plugin('begin');

	//トラックバック取得
	$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_TABLE_TRACKBACK . ' ORDER BY no DESC LIMIT :start, :limit');
	$stmt->bindValue(':start', intval($configs['_blog']['admin_size']) * $page, PDO::PARAM_INT);
	$stmt->bindValue(':limit', intval($configs['_blog']['admin_size']), PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	$trackbacks = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$trackbacks[] = array(
			'no'    => $data['no'],
			'pno'   => $data['pno'],
			'date'  => $data['date'],
			'name'  => $data['name'],
			'title' => $data['title'],
			'url'   => $data['url'],
			'text'  => $data['text'],
			'host'  => $data['host']
		);
	}
	$smarty->assign('trackbacks', $trackbacks);

	//トラックバック数取得
	$stmt = $pdo->query('SELECT COUNT(*) FROM ' . DATABASE_TABLE_TRACKBACK);
	if (!$stmt) {
		db_error($pdo->errorInfo());
	}

	$data  = $stmt->fetch(PDO::FETCH_NUM);
	$count = $data[0];

	$smarty->assign('count', $count);

	//ページ前後移動
	if ($page > 0) {
		$smarty->assign('prev', array(
			'id'   => $page - 1,
			'flag' => true
		));
	}
	if (intval(($count - 1) / $configs['_blog']['admin_size']) > $page) {
		$smarty->assign('next', array(
			'id'   => $page + 1,
			'flag' => true
		));
	}

	//ページ直接移動
	$pages = array();
	for ($i = 0; $i <= intval(($count - 1) / $configs['_blog']['admin_size']); $i++) {
		$pages[] = array(
			'id'   => $i,
			'flag' => $page == $i ? true : false
		);
	}
	$smarty->assign('pages', $pages);

	//プラグイン実行
	plugin('display');

	//データ表示
	output(TEMPLATE_ADMIN_TRACKBACK);

	//プラグイン実行
	plugin('end');

	return;
}

/* 管理者用 - トラックバック削除 */
function admin_trackback_del()
{
	global $users, $configs, $params, $pdo, $smarty;

	//アクセス元チェック
	if (isset($_SERVER['HTTP_REFERER']) and !preg_match('/^' . preg_quote(HTTPS_PATH ? HTTPS_PATH : HTTP_PATH, '/') . '/', $_SERVER['HTTP_REFERER'])) {
		error('不正なアクセスです。');
	}

	//ワンタイムトークン比較
	if (!check_token()) {
		error('不正なアクセスです。');
	}

	//入力内容チェック
	if (empty($_POST['no'])) {
		error('削除したいトラックバックを選択してください。');
	}

	$dels = array();
	foreach ($_POST['no'] as $no) {
		$dels[] = intval($no);
	}

	//プラグイン実行
	plugin('begin');

	//データ削除
	$stmt = $pdo->query('DELETE FROM ' . DATABASE_TABLE_TRACKBACK . ' WHERE no IN(' . implode(',', $dels) . ')');
	if (!$stmt) {
		db_error($pdo->errorInfo());
	}

	if (DATABASE_TYPE == 'mysql') {
		$stmt = $pdo->query('ALTER TABLE ' . DATABASE_TABLE_TRACKBACK . ' AUTO_INCREMENT = 0');
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}
	}

	$smarty->assign('message', 'No.' . implode('、', $dels) . 'のトラックバックを削除しました。');

	//ワンタイムトークン破棄
	destroy_token();

	//キャッシュ削除
	if (CACHE_ARTICLE or CACHE_VIEW) {
		$smarty->caching       = 1;
		$smarty->compile_check = false;
		$smarty->cache_dir     = CACHE_DIR;

		if (CACHE_ARTICLE) {
			$smarty->clear_cache(TEMPLATE_ARTICLE);
		}
		if (CACHE_VIEW) {
			$stmt = $pdo->query('SELECT pno FROM ' . DATABASE_TABLE_TRACKBACK . ' WHERE no IN(' . implode(',', $dels) . ') GROUP BY pno');
			if (!$stmt) {
				db_error($pdo->errorInfo());
			}

			while ($data = $stmt->fetch(PDO::FETCH_NUM)) {
				$smarty->clear_cache(TEMPLATE_VIEW, $data[0]);
			}
		}

		$smarty->caching = 0;
	}

	//プラグイン実行
	plugin('end');

	//トラックバック一覧
	admin_trackback();

	return;
}

/* 管理者用 - 設定一覧 */
function admin_config()
{
	global $users, $configs, $params, $pdo, $smarty;

	//ワンタイムトークン作成
	$smarty->assign('token', create_token());

	//作業対象ファイル取得
	$file = isset($_REQUEST['file']) ? $_REQUEST['file'] : '_blog';

	//プラグイン実行
	plugin('begin');

	//設定一覧取得
	$explain = '本体';
	$files[] = array(
		'name'    => '_blog',
		'explain' => $explain,
		'flag'    => '_blog' == $file ? true : false
	);
	$target = $explain;

	if ($dir = scandir(CONFIG_DIR)) {
		foreach ($dir as $entry) {
			if (!preg_match('/^(\w+)\.ini$/', $entry, $matches) or $matches[1] == '_blog') {
				continue;
			}

			$name    = $matches[1];
			$explain = $matches[1] . 'プラグイン';

			if ($matches[1] == $file) {
				$target = $explain;
			}

			$files[] = array(
				'name'    => $name,
				'explain' => $explain,
				'flag'    => $matches[1] == $file ? true : false
			);
		}
	} else {
		exit('設定ファイル格納ディレクトリを開けません。');
	}

	$smarty->assign(array(
		'files'  => $files,
		'target' => $target
	));

	//設定内容取得
	$flag   = false;
	$buffer = '';
	$count  = 0;
	$types  = array();
	$sizes  = array();
	$exts   = array();
	$datas  = array();

	if ($fp = fopen(CONFIG_DIR . $file . '.ini', 'r')) {
		while ($line = fgets($fp)) {
			$line = trim($line);

			if ($buffer and (preg_match('/^\[.+\]$/', $line) or preg_match('/^;.+$/', $line))) {
				$datas[] = array(
					'type' => 'config',
					'data' => '<textarea name="data[_buffer' . ++$count . ']" cols="50" rows="5">' . $buffer . '</textarea>'
				);

				$buffer = '';
			}

			if (preg_match('/^\[(.+)\]$/', $line, $matches)) {
				$flag = true;

				$datas[] = array(
					'type' => 'section',
					'data' => $matches[1]
				);
			} elseif (preg_match('/^;(.+)$/', $line, $matches)) {
				if ($matches[1] == 'exit') {
					break;
				}

				if ($flag) {
					$datas[] = array(
						'type' => 'comment',
						'data' => $matches[1]
					);
				} else {
					$define = $matches[1];

					if (preg_match('/^(.+)\s+(.+)\s+"(.+)"$/', $define, $matches)) {
						$name = $matches[1];
						$type = $matches[2];
						$ext  = $matches[3];
					} elseif (preg_match('/^(.+)\s+(.+)$/', $define, $matches)) {
						$name = $matches[1];
						$type = $matches[2];
						$ext  = '';
					} else {
						$name = '';
						$type = '';
						$ext  = '';
					}

					if ($name) {
						if (preg_match('/^(.+)\((\d+)\)$/', $type, $matches)) {
							$type = $matches[1];
							$size = $matches[2];
						} else {
							$size = '';
						}

						$types[$name] = $type;
						$sizes[$name] = $size;
						$exts[$name]  = $ext;
					}
				}
			} elseif (preg_match('/^([^\s]+)\s*=\s*(.+)$/', $line, $matches)) {
				if (!$flag) {
					continue;
				}

				$name  = $matches[1];
				$value = $matches[2];

				$value = str_replace('<', '&lt;', $value);
				$value = str_replace('>', '&gt;', $value);

				if (empty($types[$name])) {
					$buffer .= $line . "\n";
				} else {
					$data = '';

					switch ($types[$name]) {
						case 'set':
							if (preg_match('/^"(.*)"$/', $value, $matches)) {
								$value = $matches[1];
							}

							$options = explode('|', $exts[$name]);

							foreach ($options as $option) {
								list($option_key, $option_value) = explode(':', $option, 2);

								if (in_array($option_key, explode(',', $value))) {
									$checked = ' checked="checked"';
								} else {
									$checked = '';
								}

								$data .= '<input type="checkbox" name="data[' . $name . '][]" id="' . $name . '_' . $option_key . '" value="' . $option_key . '"' . $checked . ' /> <label for="' . $name . '_' . $option_key . '"">' . $option_value . '</label><br />';
							}

							break;
						case 'enum':
							if (preg_match('/^"(.*)"$/', $value, $matches)) {
								$value = $matches[1];
							}

							$options = explode('|', $exts[$name]);

							foreach ($options as $option) {
								list($option_key, $option_value) = explode(':', $option, 2);

								if ($value == $option_key) {
									$checked = ' checked="checked"';
								} else {
									$checked = '';
								}

								$data .= '<input type="radio" name="data[' . $name . ']" id="' . $name . '_' . $option_key . '" value="' . $option_key . '"' . $checked . ' /> <label for="' . $name . '_' . $option_key . '"">' . $option_value . '</label><br />';
							}

							break;
						case 'bool':
							if (empty($exts[$name])) {
								$options = array('On', 'Off');
							} else {
								$options = explode('|', $exts[$name], 2);
							}

							$data .= '<select name="data[' . $name . ']">';
							if ($value == 'On') {
								$data .= '<option value="On" selected="selected">' . $options[0] . '</option>';
								$data .= '<option value="Off">' . $options[1] . '</option>';
							} else {
								$data .= '<option value="On">' . $options[0] . '</option>';
								$data .= '<option value="Off" selected="selected">' . $options[1] . '</option>';
							}
							$data .= '</select>';

							break;
						case 'text':
							if (preg_match('/^"(.*)"$/', $value, $matches)) {
								$matches[1] = preg_replace('/" CONFIG_QUOTE "/', '&quot;', $matches[1]);
								$matches[1] = str_replace('\\\\', "\a", $matches[1]);
								$matches[1] = str_replace('\n', "\n", $matches[1]);
								$matches[1] = str_replace("\a", '\\', $matches[1]);

								$data .= '<textarea name="data[' . $name . ']" cols="50" rows="5">' . $matches[1] . '</textarea>';
							}

							break;
						case 'char':
							if (preg_match('/^"(.*)"$/', $value, $matches)) {
								$matches[1] = preg_replace('/" CONFIG_QUOTE "/', '&quot;', $matches[1]);
								$matches[1] = str_replace('\\\\', '\\', $matches[1]);

								$data .= '<input type="text" name="data[' . $name . ']" size="' . (empty($sizes[$name]) ? 80 : $sizes[$name]) . '" value="' . $matches[1] . '" />';
							}

							if (!empty($exts[$name])) {
								$data .= ' ' . $exts[$name];
							}

							break;
						case 'int':
							$data .= '<input type="text" name="data[' . $name . ']" size="' . (empty($sizes[$name]) ? 10 : $sizes[$name]) . '" value="' . $value . '" />';

							if (!empty($exts[$name])) {
								$data .= ' ' . $exts[$name];
							}

							break;
					}

					$datas[] = array(
						'type' => 'config',
						'data' => $data
					);
				}
			}
		}
		fclose($fp);
	} else {
		exit('設定ファイルを読み込めません。');
	}

	$smarty->assign('form', array(
		'file'  => $file,
		'datas' => $datas
	));

	//プラグイン実行
	plugin('display');

	//データ表示
	output(TEMPLATE_ADMIN_CONFIG);

	//プラグイン実行
	plugin('end');

	return;
}

/* 管理者用 - 設定編集 */
function admin_config_edit()
{
	global $users, $configs, $params, $pdo, $smarty;

	//アクセス元チェック
	if (isset($_SERVER['HTTP_REFERER']) and !preg_match('/^' . preg_quote(HTTPS_PATH ? HTTPS_PATH : HTTP_PATH, '/') . '/', $_SERVER['HTTP_REFERER'])) {
		error('不正なアクセスです。');
	}

	//ワンタイムトークン比較
	if (!check_token()) {
		error('不正なアクセスです。');
	}

	//入力内容チェック
	if (empty($_POST['file'])) {
		error('設定対象が指定されていません。');
	}
	if (!preg_match('/^[\w\-]+$/', $_POST['file'])) {
		error('設定対象の指定内容が正しくありません。');
	}

	//プラグイン実行
	plugin('begin');

	//設定編集
	$flag1 = false;
	$flag2 = false;
	$data  = '';
	$count = 0;
	$types = array();
	$sizes = array();
	$exts  = array();

	if ($fp = fopen(CONFIG_DIR . $_POST['file'] . '.ini', 'r')) {
		while ($line = fgets($fp)) {
			$line = trim($line);

			if (preg_match('/^\[(.+)\]$/', $line, $matches)) {
				$flag1 = true;

				$data .= $line . "\n";
			} elseif (preg_match('/^;(.+)$/', $line, $matches)) {
				if (!$flag1) {
					$define = $matches[1];

					if (preg_match('/^(.+)\s+(.+)\s+"(.+)"$/', $define, $matches)) {
						$name = $matches[1];
						$type = $matches[2];
						$ext  = $matches[3];
					} elseif (preg_match('/^(.+)\s+(.+)$/', $define, $matches)) {
						$name = $matches[1];
						$type = $matches[2];
						$ext  = '';
					} else {
						$name = '';
						$type = '';
						$ext  = '';
					}

					if ($name) {
						if (preg_match('/^(.+)\((\d+)\)$/', $type, $matches)) {
							$type = $matches[1];
							$size = $matches[2];
						} else {
							$size = '';
						}

						$types[$name] = $type;
						$sizes[$name] = $size;
						$exts[$name]  = $ext;
					}
				}
				if ($flag2) {
					$flag2 = false;
				}

				$data .= $line . "\n";
			} elseif (preg_match('/^([^\s]+)\s*=\s*(.+)$/', $line, $matches)) {
				if (!$flag1) {
					continue;
				}

				$name  = $matches[1];
				$value = $matches[2];

				if (empty($types[$name])) {
					if ($flag2) {
						continue;
					}

					if (isset($_POST['data']['_buffer' . ($count + 1)])) {
						$value = trim($_POST['data']['_buffer' . ++$count]);

						if (preg_match('/(^|\n);/', $value)) {
							error('設定内容の行頭に;を書くことはできません。');
						}

						$flag2 = true;
					} else {
						$value = $line;
					}

					$data .= trim($value) . "\n";
				} else {
					switch ($types[$name]) {
						case 'set':
							if (isset($_POST['data'][$name])) {
								$value = '"' . implode(',', $_POST['data'][$name]) . '"';
							} else {
								$value = '""';
							}
							break;
						case 'enum':
							if (isset($_POST['data'][$name])) {
								$value = '"' . $_POST['data'][$name] . '"';
							}
							break;
						case 'bool':
							if (isset($_POST['data'][$name])) {
								$value = $_POST['data'][$name];
							}
							if (!preg_match('/^(On|Off)$/', $value)) {
								error($name . 'はOn/Offで入力してください。');
							}
							break;
						case 'text':
							if (isset($_POST['data'][$name])) {
								$value = '"' . preg_replace('/"/', '" CONFIG_QUOTE "', $_POST['data'][$name]) . '"';
							}
							break;
						case 'char':
							if (isset($_POST['data'][$name])) {
								$value = '"' . preg_replace('/"/', '" CONFIG_QUOTE "', $_POST['data'][$name]) . '"';
							}
							if (preg_match('/\n/', $value)) {
								error($name . 'は一行で入力してください。');
							}
							break;
						case 'int':
							if (isset($_POST['data'][$name])) {
								$value = $_POST['data'][$name];
							}
							if (!preg_match('/^\d+$/', $value)) {
								error($name . 'は数値で入力してください。');
							}
							break;
					}

					$value = str_replace('\\', '\\\\', $value);
					$value = str_replace("\n", '\n', $value);

					$data .= $name . ' = ' . trim($value) . "\n";
				}
			} else {
				$data .= $line . "\n";
			}
		}
		fclose($fp);
	} else {
		exit('設定ファイルを読み込めません。');
	}

	if (!file_put_contents(CONFIG_DIR . $_POST['file'] . '.ini', $data)) {
		exit('設定ファイルに書き込めません。');
	}

	$smarty->assign('message', '設定ファイルを編集しました。');

	//ワンタイムトークン破棄
	destroy_token();

	//プラグイン実行
	plugin('end');

	//設定一覧
	admin_config();

	return;
}

/* 管理者用 - ステータス */
function admin_status()
{
	global $users, $configs, $params, $pdo, $smarty;

	//プラグイン実行
	plugin('begin');

	//記事数取得
	$stmt = $pdo->query('SELECT COUNT(*) FROM ' . DATABASE_TABLE_ARTICLE);
	if (!$stmt) {
		db_error($pdo->errorInfo());
	}

	$data  = $stmt->fetch(PDO::FETCH_NUM);
	$count = $data[0];

	$status['article'] = $count;

	//コメント数取得
	$stmt = $pdo->query('SELECT COUNT(*) FROM ' . DATABASE_TABLE_COMMENT);
	if (!$stmt) {
		db_error($pdo->errorInfo());
	}

	$data  = $stmt->fetch(PDO::FETCH_NUM);
	$count = $data[0];

	$status['comment'] = $count;

	//トラックバック数取得
	$stmt = $pdo->query('SELECT COUNT(*) FROM ' . DATABASE_TABLE_TRACKBACK);
	if (!$stmt) {
		db_error($pdo->errorInfo());
	}

	$data  = $stmt->fetch(PDO::FETCH_NUM);
	$count = $data[0];

	$status['trackback'] = $count;

	//アップロードファイルサイズ取得
	if ($dir = scandir(FILE_DIR)) {
		$file = 0;
		foreach ($dir as $entry) {
			if (!is_file(FILE_DIR . $entry)) {
				continue;
			}

			$file += filesize(FILE_DIR . $entry);
		}

		$status['file'] = intval($file / 1024);
	} else {
		exit('アップロードファイル格納ディレクトリを開けません。');
	}

	$smarty->assign('status', $status);

	//プラグイン実行
	plugin('display');

	//データ表示
	output(TEMPLATE_ADMIN_STATUS);

	//プラグイン実行
	plugin('end');

	return;
}

/* 管理者用 - ログアウト */
function admin_logout()
{
	global $users, $configs, $params, $pdo, $smarty;

	//プラグイン実行
	plugin('begin');

	//ログイン情報クリア
	if (isset($_COOKIE['session'])) {
		unlink(LOGIN_DIR . $_COOKIE['session']);

		if (isset($_SERVER['HTTPS']) and $_SERVER['HTTPS'] != 'off') {
			$info = parse_url(HTTPS_PATH);
		} else {
			$info = parse_url(HTTP_PATH);
		}

		setcookie('session', '', 0, $info['path']);
	}
	$_SESSION['login'] = false;

	//プラグイン実行
	plugin('end');

	//管理画面
	admin();

	return;
}

/* エラー表示 */
function error($message)
{
	global $users, $configs, $params, $pdo, $smarty;

	$smarty->assign('message', $message);

	output(TEMPLATE_ERROR);

	exit;
}

/* データベースエラー表示 */
function db_error($info)
{
	global $users, $configs, $params, $pdo, $smarty;

	list($state, $code, $message) = $info;

	exit('Error: ' . $message);
}

/* トラックバック受信エラー表示 */
function tb_error($message)
{
	global $users, $configs, $params, $pdo, $smarty;

	header('Content-Type: text/xml');

	echo "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n";
	echo "<response>\n";
	echo "<error>1</error>\n";
	echo "<message>$message</message>\n";
	echo "</response>\n";

	exit;
}

/* データ出力 */
function output($template, $id = null, $content = null)
{
	global $users, $configs, $params, $pdo, $smarty;

	if ($content == null) {
		$content = $users['content'];
	}

	header('Content-Type: ' . $content . '; charset=UTF-8');

	if ($users['type'] == 'mobile') {
		header('Pragma: no-cache');
		header('Cache-Control: no-cache');

		if ($smarty->template_exists(TEMPLATE_MOBILE . $template)) {
			$template = TEMPLATE_MOBILE . $template;
		}
	}

	if (PICTOGRAM_MODE) {
		echo convert_pictogram($smarty->fetch($template, $id));
	} else {
		$smarty->display($template, $id);
	}

	return;
}

/* タグ整形 */
function create_tag($string)
{
	if (!$string) {
		return;
	}

	if (REWRITE_MODE) {
		$script_url = preg_replace('/\/$/', '', HTTP_PATH);
	} else {
		$script_url = HTTP_PATH . MAIN_FILE;
	}

	$tags = array();
	foreach (explode(',', $string) as $tag) {
		$tags[] = '<a href="' . $script_url . '/search/tag/' . urlencode($tag) . '">' . htmlspecialchars($tag, ENT_QUOTES) . '</a>';
	}

	return implode(' ', $tags);
}

/* ワンタイムトークン作成 */
function create_token()
{
	global $users;

	if ($users['type'] == 'mobile') {
		return;
	}

	$token = md5(uniqid(rand(), true));

	$_SESSION['token'] = $token;

	return $token;
}

/* ワンタイムトークン比較 */
function check_token()
{
	global $users;

	if ($users['type'] == 'mobile') {
		return true;
	}

	if (isset($_REQUEST['token']) and isset($_SESSION['token']) and $_REQUEST['token'] == $_SESSION['token']) {
		return true;
	} else {
		return false;
	}
}

/* ワンタイムトークン破棄 */
function destroy_token()
{
	global $users;

	if ($users['type'] == 'mobile') {
		return;
	}

	$_SESSION['token'] = '';

	return;
}

/* 不正データ削除 */
function sanitize($data)
{
	if (is_array($data)) {
		return array_map('sanitize', $data);
	}

	return str_replace("\0", '', $data);
}

/* 改行コード統一 */
function unify($data)
{
	if (is_array($data)) {
		return array_map('unify', $data);
	}

	$data = preg_replace("/\r?\n/", "\r", $data);
	$data = preg_replace("/\r/", "\n", $data);

	return $data;
}

/* 文字コード変換 */
function convert($data)
{
	if (is_array($data)) {
		return array_map('convert', $data);
	}

	return mb_convert_encoding($data, 'UTF-8', 'UTF-8,SJIS-WIN');
}

/* 絵文字コード統一 */
function unify_pictogram($data)
{
	global $users;

	if (is_array($data)) {
		return array_map('unify_pictogram', $data);
	}

	if ($users['career'] == 'docomo' or $users['career'] == 'softbank' or $users['career'] == 'au') {
		if ($users['career'] == 'docomo') {
			$career  = MPC_FROM_FOMA;
			$charset = MPC_FROM_CHARSET_UTF8;
		} elseif ($users['career'] == 'softbank') {
			$career  = MPC_FROM_SOFTBANK;
			$charset = MPC_FROM_CHARSET_UTF8;
		} elseif ($users['career'] == 'au') {
			$career  = MPC_FROM_EZWEB;
			$charset = MPC_FROM_CHARSET_SJIS;
		}
		$mpc = MobilePictogramConverter::factory($data, $career, $charset, MPC_FROM_OPTION_RAW);
	} else {
		$mpc = MobilePictogramConverter::factory($data, MPC_FROM_FOMA, MPC_FROM_CHARSET_UTF8, MPC_FROM_OPTION_WEB);
	}
	$mpc->setImagePath(PICTOGRAM_IMAGE_DIR);

	return $mpc->Convert(MPC_TO_FOMA, MPC_TO_OPTION_RAW);
}

/* キャリアに応じた絵文字に変換 */
function convert_pictogram($data)
{
	global $users;

	if (is_array($data)) {
		return array_map('convert_pictogram', $data);
	}

	$mpc = MobilePictogramConverter::factory($data, MPC_FROM_FOMA, MPC_FROM_CHARSET_UTF8, MPC_FROM_OPTION_RAW);
	$mpc->setImagePath(PICTOGRAM_IMAGE_DIR);

	$string = $mpc->autoConvert();

	if ($users['career'] != 'docomo' and $users['career'] != 'softbank' and $users['career'] != 'au') {
		$string = escape_pictogram($string);
		$string = str_replace('$', "\a", $string);
		$string = preg_replace('/(value="([^"\\\\]|\\\\.)*")/e', 'unescape_pictogram("$1","text")', $string);
		$string = preg_replace('/(<textarea [^>]+>[^<]+<\/textarea>)/e', 'unescape_pictogram("$1","text")', $string);
		$string = str_replace("\a", '$', $string);
		$string = unescape_pictogram($string);
	}

	return $string;
}

/* 絵文字コード削除 */
function except_pictogram($data)
{
	global $users;

	if (is_array($data)) {
		return array_map('except_pictogram', $data);
	}

	$mpc = MobilePictogramConverter::factory($data, MPC_FROM_FOMA, MPC_FROM_CHARSET_UTF8, MPC_FROM_OPTION_RAW);
	$mpc->setImagePath(PICTOGRAM_IMAGE_DIR);

	return $mpc->Except();
}

/* 絵文字をエスケープ */
function escape_pictogram($data)
{
	return preg_replace('/<img src="([^"]+)" alt="" border="0" width="12" height="12" \/>/', '[&&$1]', $data);
}

/* 絵文字をアンエスケープ */
function unescape_pictogram($data, $type = 'html')
{
	if ($type == 'html') {
		return preg_replace('/\[&&([^\]]+)\]/', '<img src="$1" alt="" border="0" width="12" height="12" />', $data);
	} else {
		return preg_replace('/\[&&([^\]]+)\]/e', '"&amp;#".basename("$1",".gif").";"', $data);
	}
}

?>
